@@ -19,11 +19,86 @@ __all__ = ["relative_sensitivity", "calculate_sensitivity"]
 log = logging.getLogger(__name__)
 
 
+def _relative_sensitivity(
+    n_on,
+    n_off,
+    alpha,
+    min_significance=5,
+    min_signal_events=10,
+    min_excess_over_background=0.05,
+    significance_function=li_ma_significance,
+):
+    if np.isnan(n_on) or np.isnan(n_off):
+        return np.nan
+
+    if n_on < 0 or n_off < 0:
+        raise ValueError(f'n_on and n_off must be positive, got {n_on}, {n_off}')
+
+    n_background = n_off * alpha
+    n_signal = n_on - n_background
+
+    if n_signal <= 0:
+        return np.inf
+
+    def equation(relative_flux):
+        n_on = n_signal * relative_flux + n_background
+        s = significance_function(n_on, n_off, alpha)
+        return s - min_significance
+
+    try:
+        # brentq needs a lower and an upper bound
+        # we will use the simple, analytically  solvable significance formula and scale it
+        # with 10 to be sure it's above the Li and Ma solution
+        # so rel * n_signal / sqrt(n_background) = target_significance
+        if n_off > 1:
+            relative_flux_naive = min_significance * np.sqrt(n_background) / n_signal
+            upper_bound = 10 * relative_flux_naive
+            lower_bound = 0.01 * relative_flux_naive
+        else:
+            upper_bound = 100
+            lower_bound = 1e-4
+
+        relative_flux = brentq(equation, lower_bound, upper_bound)
+
+    except (RuntimeError, ValueError) as e:
+        log.warn(
+            "Could not calculate relative significance for"
+            f" n_signal={n_signal:.1f}, n_off={n_off:.1f}, returning nan {e}"
+        )
+        return np.nan
+
+    # less than min_sigma
+    if relative_flux > 1:
+        return np.nan
+
+    # scale to achieved flux level
+    n_signal = n_signal * relative_flux
+    min_excess = min_excess_over_background * n_background
+    min_signal = max(min_signal_events, min_excess)
+
+    # if we violate the min signal events condition,
+    # increase flux until we meet the requirement
+    if n_signal < min_signal:
+        scale = min_signal / n_signal
+    else:
+        scale = 1.0
+
+    return relative_flux * scale
+
+
+_relative_sensitivity_vectorized = np.vectorize(
+    _relative_sensitivity,
+    excluded=['significance_function']
+)
+
+
 def relative_sensitivity(
     n_on,
     n_off,
     alpha,
-    target_significance=5,
+    min_significance=5,
+    min_signal_events=10,
+    min_excess_over_background=0.05,
     significance_function=li_ma_significance,
 ):
     """
@@ -42,17 +117,31 @@ def relative_sensitivity(
     The reference time should be incorporated by appropriately weighting the events
     before calculating ``n_on`` and ``n_off``.
 
+    All input values with the exception of ``significance_function``
+    must be broadcastable to a single, common shape.
+
     Parameters
     ----------
     n_on: int or array-like
         Number of signal-like events for the on observations
     n_off: int or array-like
         Number of signal-like events for the off observations
-    alpha: float
+    alpha: float or array-like
         Scaling factor between on and off observations.
         1 / number of off regions for wobble observations.
-    significance: float
+    min_significance: float or array-like
         Significance necessary for a detection
+    min_signal_events: int or array-like
+        Minimum number of signal events required.
+        The relative flux will be scaled up from the one yielding ``min_significance``
+        if this condition is violated.
+    min_excess_over_background: float or array-like
+        Minimum number of signal events expressed as the proportion of the
+        background.
+        So the required number of signal events will be
+        ``min_excess_over_background * alpha * n_off``.
+        The relative flux will be scaled up from the one yielding ``min_significance``
+        if this condition is violated.
     significance_function: function
         A function f(n_on, n_off, alpha) -> significance in sigma
         Used to calculate the significance, default is the Li&Ma
@@ -62,53 +151,24 @@ def relative_sensitivity(
         The Astrophysical Journal 272 (1983): 317-324.
         Formula (17)
     """
-    if np.isnan(n_on) or np.isnan(n_off):
-        return np.nan
-
-    if n_on < 0 or n_off < 0:
-        raise ValueError(f'n_on and n_off must be positive, got {n_on}, {n_off}')
-
-    n_background = n_off * alpha
-    n_signal = n_on - n_background
-
-    if n_signal <= 0:
-        return np.inf
-
-    def equation(relative_flux):
-        n_on = n_signal * relative_flux + n_background
-        s = significance_function(n_on, n_off, alpha)
-        return s - target_significance
-
-    try:
-        # brentq needs a lower and an upper bound
-        # we will use the simple, analytically  solvable significance formula and scale it
-        # with 10 to be sure it's above the Li and Ma solution
-        # so rel * n_signal / sqrt(n_background) = target_significance
-        if n_off > 1:
-            relative_flux_naive = target_significance * np.sqrt(n_background) / n_signal
-            upper_bound = 10 * relative_flux_naive
-            lower_bound = 0.01 * relative_flux_naive
-        else:
-            upper_bound = 100
-            lower_bound = 1e-4
-
-        relative_flux = brentq(equation, lower_bound, upper_bound)
-
-    except (RuntimeError, ValueError) as e:
-        log.warn(
-            "Could not calculate relative significance for"
-            f" n_signal={n_signal:.1f}, n_off={n_off:.1f}, returning nan {e}"
-        )
-        return np.nan
-
-    return relative_flux
+    return _relative_sensitivity_vectorized(
+        n_on=n_on,
+        n_off=n_off,
+        alpha=alpha,
+        min_significance=min_significance,
+        min_signal_events=min_signal_events,
+        min_excess_over_background=min_excess_over_background,
+        significance_function=significance_function,
+    )
 
 
 def calculate_sensitivity(
     signal_hist,
     background_hist,
     alpha,
-    target_significance=5,
+    min_significance=5,
+    min_signal_events=10,
+    min_excess_over_background=0.05,
     significance_function=li_ma_significance,
 ):
     """
@@ -138,8 +198,19 @@ def calculate_sensitivity(
         See ``pyirf.binning.create_histogram_table``
     alpha: float
         Size ratio of signal region to background region
-    target_significance: float
-        Required significance
+    min_significance: float
+        Significance necessary for a detection
+    min_signal_events: int
+        Minimum number of signal events required.
+        The relative flux will be scaled up from the one yielding ``min_significance``
+        if this condition is violated.
+    min_excess_over_background: float
+        Minimum number of signal events expressed as the proportion of the
+        background.
+        So the required number of signal events will be
+        ``min_excess_over_background * alpha * n_off``.
+        The relative flux will be scaled up from the one yielding ``min_significance``
+        if this condition is violated.
     significance_function: callable
         A function with signature (n_on, n_off, alpha) -> significance.
         Default is the Li & Ma likelihood ratio test.
@@ -155,17 +226,23 @@ def calculate_sensitivity(
     """
     check_histograms(signal_hist, background_hist)
 
+    n_on = signal_hist["n_weighted"] + alpha * background_hist["n_weighted"]
+
+    # convert any quantities to arrays,
+    # since quantitites don't work with vectorize
+    n_on = u.Quantity(n_on, copy=False).to_value(u.one)
+    n_off = u.Quantity(background_hist["n_weighted"], copy=False).to_value(u.one)
+
     # calculate sensitivity in each bin
-    rel_sens = np.array([
-        relative_sensitivity(
-            n_on=n_signal + alpha * n_background,
-            n_off=n_background,
-            alpha=alpha,
-        )
-        for n_signal, n_background in zip(
-            signal_hist["n_weighted"], background_hist["n_weighted"]
-        )
-    ])
+    rel_sens = relative_sensitivity(
+        n_on=n_on,
+        n_off=n_off,
+        alpha=alpha,
+        min_significance=min_significance,
+        min_signal_events=min_signal_events,
+        min_excess_over_background=min_excess_over_background,
+        significance_function=significance_function,
+    )
 
     # fill output table
     s = QTable()
@@ -174,26 +251,19 @@ def calculate_sensitivity(
         s[k] = signal_hist[k]
 
     s["n_signal"] = signal_hist["n"] * rel_sens
-    s["n_background"] = background_hist["n"]
     s["n_signal_weighted"] = signal_hist["n_weighted"] * rel_sens
+    s["n_background"] = background_hist["n"]
     s["n_background_weighted"] = background_hist["n_weighted"]
 
-    # perform safety checks
-    # we use the number of signal events at the flux level that yields
-    # the target significance
-    # safety checks according to the IRF document
-    # at least ten signal events and the number of signal events
-    # must be larger then five percent of the remaining background
-    min_excess = 0.05 * alpha * s["n_background_weighted"]
+    # copy also "n_proton" / "n_electron_weighted" etc. if available
+    for k in filter(lambda c: c.startswith('n_') and c != 'n_weighted', background_hist.colnames):
+        s[k] = background_hist[k]
 
-    s['failed_checks'] = (
-        (s['n_signal_weighted'] < 10)
-        | ((s['n_signal_weighted'] < min_excess) << 1)
-        | ((rel_sens > 1) << 2)
+    s["significance"] = significance_function(
+        n_on=s["n_signal_weighted"] + alpha * s["n_background_weighted"],
+        n_off=s["n_background_weighted"],
+        alpha=alpha,
     )
-
-    rel_sens[s['failed_checks'] > 0] = np.nan
-
     s["relative_sensitivity"] = rel_sens
 
     return s
@@ -223,7 +293,7 @@ def estimate_background(
         and inside ``background_radius`` from the center of the FOV
         Required columns for this function:
         - `reco_energy`,
-        - `source_fov_offset`.
+        - `reco_source_fov_offset`.
     reco_energy_bins: astropy.units.Quantity[energy]
         Desired bin edges in reconstructed energy for the background rate
     theta_cuts: astropy.table.QTable
@@ -236,7 +306,7 @@ def estimate_background(
         Maximum distance from the fov center for background events to be taken into account
     '''
     bg = create_histogram_table(
-        events[events['source_fov_offset'] < background_radius],
+        events[events['reco_source_fov_offset'] < background_radius],
         reco_energy_bins,
         key='reco_energy',
     )
@@ -255,7 +325,7 @@ def estimate_background(
         / cone_solid_angle(background_radius)
     ).to_value(u.one)
 
-    for key in ['n', 'n_weighted']:
+    for key in filter(lambda col: col.startswith('n'), bg.colnames):
         # *= not possible due to upcast from int to float
         bg[key] = bg[key] * size_ratio / alpha
 
